*==============================================================================
* Class:				CHECKHEALTHIDS
* Based On: 			Custom
* Purpose:				Check Canadian health IDs (PHNs); see About method for details
* Author:				Tamar Granor
* Copyright:			(c) 2010 Tamar E. Granor, Ph.D.
* Last revision:		01/22/10
*==============================================================================
DEFINE CLASS HealthIDCheckDigit AS Custom

DIMENSION aCheckMethods[1]

PROCEDURE Init

LOCAL lOpen

* Grab the driving data
lOpen = USED("CheckDigitCode")
SELECT cProvince, mCode, nIDLen ;
	FROM CheckDigitCode ;
	WHERE lRecip ;
	INTO ARRAY This.aCheckMethods

IF NOT lOpen
	USE IN CheckDigitCode
ENDIF 
	
ENDPROC

PROCEDURE CheckID(cID, cProv)

LOCAL lReturn, nRow

* Find the right method and call it
nRow = ASCAN(This.aCheckMethods, UPPER(cProv), -1, -1, 1, 8)
IF nRow = 0
	* Return .F. if the province code is no good.
	lReturn = .F.
ELSE
	* First check length
	IF LEN(ALLTRIM(cID)) = This.aCheckMethods[nRow, 3]
		cCode = This.aCheckMethods(nRow, 2)
		lReturn = EXECSCRIPT(cCode, ALLTRIM(cID), This)
	ELSE
		lReturn = .F.
	ENDIF
ENDIF

RETURN lReturn
ENDPROC

PROCEDURE DoubleOddUnitsFromTen
* Double odd digits, sum, then subtract units from 10
LPARAMETERS cID

LOCAL nTotal, nDigits, nDigit, nUnits, nCheckDigit, lReturn, aMultipliers[1]

nDigits = LEN(TRIM(cID))
DIMENSION aMultipliers[nDigits]
FOR nDigit = 1 TO nDigits-1
	IF MOD(nDigit,2) = 1
		aMultipliers[nDigit] = 2
	ELSE
		aMultipliers[nDigit] = 1
	ENDIF
ENDFOR 
aMultipliers[nDigits] = 0

nTotal = This.MultiplyAndAddDigits(cID, @aMultipliers)

nUnits = MOD(nTotal, 10)
IF nUnits = 0
	nUnits = 10
ENDIF 
nCheckDigit = 10-nUnits

lReturn = (nCheckDigit = VAL(SUBSTR(cID, nDigits, 1)))
RETURN lReturn
ENDPROC

PROCEDURE DoubleEvenUnitsFromTen
* Double even digits, sum, then subtract units from 10
LPARAMETERS cID

LOCAL nTotal, nDigits, nDigit, nUnits, nCheckDigit, lReturn, aMultipliers[1]

nDigits = LEN(TRIM(cID))
DIMENSION aMultipliers[nDigits]
FOR nDigit = 1 TO nDigits-1
	IF MOD(nDigit,2) = 1
		aMultipliers[nDigit] = 1
	ELSE
		aMultipliers[nDigit] = 2
	ENDIF
ENDFOR 
aMultipliers[nDigits] = 0

nTotal = This.MultiplyAndAddDigits(cID, @aMultipliers)

nUnits = MOD(nTotal, 10)
IF nUnits = 0
	nUnits = 10
ENDIF 
nCheckDigit = 10-nUnits

lReturn = (nCheckDigit = VAL(SUBSTR(cID, nDigits, 1)))
RETURN lReturn
ENDPROC

PROCEDURE MultiplyAndAddDigits(cID, aMultipliers)
* Multiply digits by the specified factors, then add all digits 

LOCAL nDigits, nDigit, nResult, nProduct

nDigits = ALEN(aMultipliers)
nResult = 0
FOR nDigit = 1 TO nDigits
	nProduct = VAL(SUBSTR(cID,nDigit,1)) * aMultipliers[nDigit]
	nResult = nResult + MOD(nProduct, 10) + FLOOR(nProduct/10)
ENDFOR

RETURN nResult

PROCEDURE Mod11Check(cID, cMultipliers)
* Multiply digits by the specified factors, add results and return
* result mod 11

LOCAL nTotal, nResult

nTotal = This.MultiplyAndAdd(cID, cMultipliers)

nResult = MOD(nTotal, 11)

RETURN nResult
ENDPROC

PROCEDURE MultiplyAndAdd(cID, cMultipliers)
* Multiply digits by the specified factors and sum results

LOCAL nDigits, nDigit, nProduct, nTotal, aMultipliers[1]

nDigits = ALINES(aMultipliers, cMultipliers, ",")
FOR nDigit = 1 TO nDigits
	aMultipliers[nDigit] = VAL(aMultipliers[nDigit])
ENDFOR

nTotal = 0

FOR nDigit = 1 TO nDigits
	nProduct = VAL(SUBSTR(cID, nDigit, 1)) * aMultipliers[nDigit]
	nTotal = nTotal + nProduct
ENDFOR

RETURN nTotal
ENDPROC 

PROCEDURE About
* Documentation

TEXT
This class performs validation of Canadian HealthIDs. It's data-driven,
in order to better accomodate future changes. The class itself contains a 
driver method (CheckID) and generic methods for an assortment of manipulations.
The class works with the CheckDigitCode table, which contains the actual
code for each province. If a province's rules change, the code in the mCode
method of CheckDigitCode for that province should be changed. Class code
should only change to fix bugs or handle new generic cases.
ENDTEXT

ENDDEFINE
