* program : fjlistener.prg
* purpose : provides a report listener that allows rendering text in
*           full justify alignment.
* author : cesar chalom and victor espinoza
* class based on article "listening to a report" by doug hennig
* http://msdn2.microsoft.com/en-us/library/ms947682.aspx
DEFINE CLASS fulljustifylistener AS _reportlistener OF ;
    ADDBS(HOME(1)) + "ffc\_reportlistener.vcx"
  ogdigraphics = NULL
  DIMENSION arecords[1]
  * doug hennig - if listenertype hasn't already been set, set it based on whether the report
  * is being printed or previewed.
  FUNCTION LOADREPORT
    WITH THIS
      DO CASE
        CASE .LISTENERTYPE <> -1
        CASE .COMMANDCLAUSES.PREVIEW
          .LISTENERTYPE = 1
        CASE .COMMANDCLAUSES.outputto = 1
          .LISTENERTYPE = 0
      ENDCASE
    ENDWITH
    DODEFAULT()
  ENDFUNC
  * before we run the report, we need to make sure that gdiplusx
  * was initialized, and create a new graphics object
  FUNCTION BEFOREREPORT
    DODEFAULT()
    WITH THIS
      * check if we already have the "system" object in "_screen"
      IF NOT PEMSTATUS(_SCREEN,"system",5)
        DO LOCFILE("system.app")
      ENDIF
      .ogdigraphics = _SCREEN.SYSTEM.drawing.graphics.new()
      .setfrxdatasession() && switches to the frx cursor's datasession
      DIMENSION .arecords(RECCOUNT(), 13)
      SCAN FOR "<fj>" $ UPPER(USER)
        .arecords(RECNO(), 13) = "fj"
      ENDSCAN
      .resetdatasession() && restores the datasession id to the one the listener is in
    ENDWITH
  ENDFUNC
  FUNCTION BEFOREBAND(nbandobjcode, nfrxrecno)
    THIS.sharedgdiplusgraphics = THIS.GDIPLUSGRAPHICS
    THIS.ogdigraphics.handle = THIS.sharedgdiplusgraphics
    DODEFAULT(nbandobjcode, nfrxrecno)
  ENDFUNC
  PROCEDURE RENDER(tnfrxrecno,;
      tnleft,tntop,tnwidth,tnheight,;
      nobjectcontinuationtype, ;
      ccontentstoberendered, gdiplusimage)
    IF VARTYPE(THIS.arecords(tnfrxrecno,13)) = "c" AND ;
        THIS.arecords(tnfrxrecno,13) == "fj"
      LOCAL lctext
      lctext = THIS.arecords(tnfrxrecno,1)
      THIS.ogdigraphics.handle = THIS.GDIPLUSGRAPHICS
      WITH _SCREEN.SYSTEM.drawing
        * create a gdi+ rectangle which specifies where on the
        * surface we're drawing the text.
        LOCAL lorectf AS xfcrectanglef
        lorectf = .rectanglef.new(tnleft, tntop, tnwidth, tnheight)
        * create a font object based on the report original settings
        LOCAL lofont AS xfcfont
        lofont = .FONT.new(THIS.arecords(tnfrxrecno,2) ;
          , THIS.arecords(tnfrxrecno,4), THIS.arecords(tnfrxrecno,3) ;
          , .graphicsunit.POINT)
        * if we have an opaque background set for the text, then draw a rectangle
        * using the background chosen background color
        IF THIS.arecords[tnfrxrecno,8] <> 0 && alpha
          * retrieve colors for the background
          LOCAL lnred, lngreen, lnblue, lnalpha
          lnred = THIS.arecords[tnfrxrecno,5]
          lngreen = THIS.arecords[tnfrxrecno,6]
          lnblue = THIS.arecords[tnfrxrecno,7]
          lnalpha = THIS.arecords[tnfrxrecno,8]
          * create a solid brush that will be used to draw the background
          LOCAL lobackbrush AS xfcsolidbrush
          lobackbrush = .solidbrush.new(;
            .COLOR.fromargb(lnalpha,lnred, lngreen, lnblue))
          * draw the background rectangle
          THIS.ogdigraphics.fillrectangle(lobackbrush, tnleft, tntop, tnwidth, tnheight)
        ENDIF
        * retieve colors for the text
        lnred = THIS.arecords[tnfrxrecno,9]
        lngreen = THIS.arecords[tnfrxrecno,10]
        lnblue = THIS.arecords[tnfrxrecno,11]
        lnalpha = THIS.arecords[tnfrxrecno,12]
        * create a solid brush that will be used to draw the text
        LOCAL lotextbrush AS xfcsolidbrush
        lotextbrush = .solidbrush.new(;
          .COLOR.fromargb(lnalpha,lnred, lngreen, lnblue))
        * finally, draw the text in fulljustified mode.
        THIS.ogdigraphics.drawstringjustified(lctext, lofont, lotextbrush, lorectf)
      ENDWITH
    ELSE
      * if we're not drawing a full justified string,
      * let vfp draw the text as usual.
      DODEFAULT(tnfrxrecno, tnleft, tntop, tnwidth, tnheight, ;
        nobjectcontinuationtype, ccontentstoberendered, gdiplusimage)
    ENDIF
    * since we already drew the text, we don't want the default
    * behavior to occur.
    NODEFAULT
  ENDPROC
  FUNCTION EVALUATECONTENTS(tnfrxrecno, toobjproperties)
    * get the frx data
    THIS.arecords[tnfrxrecno,1] = toobjproperties.TEXT
    THIS.arecords[tnfrxrecno,2] = toobjproperties.FONTNAME
    THIS.arecords[tnfrxrecno,3] = toobjproperties.fontstyle
    THIS.arecords[tnfrxrecno,4] = toobjproperties.FONTSIZE
    THIS.arecords[tnfrxrecno,5] = toobjproperties.fillred
    THIS.arecords[tnfrxrecno,6] = toobjproperties.fillgreen
    THIS.arecords[tnfrxrecno,7] = toobjproperties.fillblue
    THIS.arecords[tnfrxrecno,8] = toobjproperties.fillalpha
    THIS.arecords[tnfrxrecno,9] = toobjproperties.penred
    THIS.arecords[tnfrxrecno,10] = toobjproperties.pengreen
    THIS.arecords[tnfrxrecno,11] = toobjproperties.penblue
    THIS.arecords[tnfrxrecno,12] = toobjproperties.penalpha
  ENDFUNC
ENDDEFINE
